@@ -46,6 +46,7 @@ struct packet_cb {
 	u64 nonce;
 	struct noise_keypair *keypair;
 	atomic_t state;
+	u32 mtu;
 	u8 ds;
 };
 #define PACKET_PEER(skb) (((struct packet_cb *)skb->cb)->keypair->entry.peer)
@@ -63,8 +64,10 @@ static inline __be16 skb_examine_untrusted_ip_hdr(struct sk_buff *skb)
 
 static inline void skb_reset(struct sk_buff *skb)
 {
-	skb_scrub_packet(skb, false);
+	const int pfmemalloc = skb->pfmemalloc;
+	skb_scrub_packet(skb, true);
 	memset(&skb->headers_start, 0, offsetof(struct sk_buff, headers_end) - offsetof(struct sk_buff, headers_start));
+	skb->pfmemalloc = pfmemalloc;
 	skb->queue_mapping = 0;
 	skb->nohdr = 0;
 	skb->peeked = 0;
